@@ -11,8 +11,11 @@
 #ifndef INCLUDED_GR_RUNTIME_BLOCK_H
 #define INCLUDED_GR_RUNTIME_BLOCK_H
 
+#include <memory>
+
 #include <gnuradio/api.h>
 #include <gnuradio/basic_block.h>
+#include <gnuradio/buffer_type.h>
 #include <gnuradio/config.h>
 #include <gnuradio/logger.h>
 #include <gnuradio/tags.h>
@@ -514,6 +517,55 @@ public:
      */
     void set_min_output_buffer(int port, long min_output_buffer);
 
+    /*!
+     * \brief DEPRECATED Configure the timer set when input is blocked \p port.
+     *
+     * \details
+     * This is an advanced/experimental feature and might be removed in a future
+     * version. Calling this can affect some fundamental assumptions about the
+     * system behavior and
+     * performance.
+     *
+     * In the TPB scheduler, when a block has no work to do because there
+     * is no data at it inputs, it sets a timer and tries again after a
+     * period of time.  The default is 250 ms, but this can be configured
+     * differently per block when necessary
+     *
+     * \param timer_value_ms the timer value in milliseconds
+     */
+    void set_blkd_input_timer_value(unsigned int timer_value_ms);
+
+    /*!
+     * \brief DEPRECATED Returns timer value set when input is blocked
+     */
+    unsigned int blkd_input_timer_value();
+
+
+    /*!
+     * \brief Allocate the block_detail and necessary output buffers for this
+     * block.
+     */
+    void allocate_detail(int ninputs,
+                         int noutputs,
+                         const std::vector<int>& downstream_max_nitems_vec,
+                         const std::vector<uint64_t>& downstream_lcm_nitems_vec,
+                         const std::vector<uint32_t>& downstream_max_out_mult_vec);
+
+    // --------------- Custom buffer-related functions -------------
+
+    /*!
+     * \brief Replace the block's buffer with a new one owned by the block_owner
+     * parameter
+     *
+     * \details
+     * This function is used to replace the buffer on the specified output port
+     * of the block with a new buffer that is "owned" by the specified block. This
+     * function will only be called if a downstream block is using a custom buffer
+     * that is incompatible with the default buffer type created by this block.
+     *
+     */
+    buffer_sptr replace_buffer(size_t src_port, size_t dst_port, block_sptr block_owner);
+
     // --------------- Performance counter functions -------------
 
     /*!
@@ -646,7 +698,7 @@ public:
      * \brief Checks if this block is already exporting perf. counters
      * to ControlPort.
      */
-    bool is_pc_rpc_set() { return d_pc_rpc_set; }
+    bool is_pc_rpc_set() const { return d_pc_rpc_set; }
 
     /*!
      * \brief If the block calls this in its constructor, it's
@@ -716,7 +768,7 @@ public:
      * \li fatal
      * \li emerg
      */
-    void set_log_level(std::string level) override;
+    void set_log_level(const std::string& level) override;
 
     /*!
      * \brief Get the logger's output level
@@ -763,7 +815,7 @@ protected:
      *
      * \param which_output an integer of which output stream to attach the tag
      * \param abs_offset   a uint64 number of the absolute item number
-     *                     assicated with the tag. Can get from nitems_written.
+     *                     associated with the tag. Can get from nitems_written.
      * \param key          the tag key as a PMT symbol
      * \param value        any PMT holding any value for the given key
      * \param srcid        optional source ID specifier; defaults to PMT_F
@@ -795,7 +847,7 @@ protected:
      *
      * \param which_input an integer of which input stream to remove the tag from
      * \param abs_offset   a uint64 number of the absolute item number
-     *                     assicated with the tag. Can get from nitems_written.
+     *                     associated with the tag. Can get from nitems_written.
      * \param key          the tag key as a PMT symbol
      * \param value        any PMT holding any value for the given key
      * \param srcid        optional source ID specifier; defaults to PMT_F
@@ -868,7 +920,7 @@ protected:
      *
      * \details
      *
-     * This opperates much like get_tags_in_range but allows us to
+     * This operates much like get_tags_in_range but allows us to
      * work within the current window of items. Item range is
      * therefore within the possible range of 0 to
      * ninput_items[whic_input].
@@ -909,15 +961,27 @@ protected:
 
     void enable_update_rate(bool en);
 
+    /*!
+     * \brief Allocate a buffer for the given output port of this block. Note
+     * that the downstream max number of items must be passed in to this
+     * function for consideration.
+     */
+    buffer_sptr allocate_buffer(size_t port,
+                                int downstream_max_nitems,
+                                uint64_t downstream_lcm_nitems,
+                                uint32_t downstream_max_out_mult);
+
     std::vector<long> d_max_output_buffer;
     std::vector<long> d_min_output_buffer;
 
+    unsigned int d_blkd_input_timer_value = 250;
+
     /*! Used by block's setters and work functions to make
      * setting/resetting of parameters thread-safe.
      *
      * Used by calling gr::thread::scoped_lock l(d_setlock);
      */
-    gr::thread::mutex d_setlock;
+    mutable gr::thread::mutex d_setlock;
 
     // These are really only for internal use, but leaving them public avoids
     // having to work up an ever-varying list of friend GR_RUNTIME_APIs
